@@ -4,14 +4,40 @@
  * Copyright (C) 2001, 2002 IBM Corporation. All Rights Reserved.
  * Author: Eric Mader <mader@jtcsv.com>
  *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use, copy,
+ * modify, merge, publish, distribute, and/or sell copies of the
+ * Software, and to permit persons to whom the Software is furnished
+ * to do so, provided that the above copyright notice(s) and this
+ * permission notice appear in all copies of the Software and that
+ * both the above copyright notice(s) and this permission notice
+ * appear in supporting documentation.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE
+ * COPYRIGHT HOLDER OR HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR
+ * ANY CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY
+ * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
+ * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
+ * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
+ * OF THIS SOFTWARE.
+ *
+ * Except as contained in this notice, the name of a copyright holder
+ * shall not be used in advertising or otherwise to promote the sale,
+ * use or other dealings in this Software without prior written
+ * authorization of the copyright holder.
  */
 
 #ifndef __INDIC_OT_H__
 #define __INDIC_OT_H__
 
-#include <freetype/freetype.h>
 #include <pango/pango-glyph.h>
 #include <pango/pango-types.h>
+#include "mprefixups.h"
 
 G_BEGIN_DECLS
 
@@ -47,7 +73,8 @@ typedef enum
   CC_DEPENDENT_VOWEL      = 7,
   CC_VIRAMA               = 8,
   CC_ZERO_WIDTH_MARK      = 9,
-  CC_COUNT                = 10
+  CC_AL_LAKUNA            = 10,
+  CC_COUNT                = 11
 } IndicOTCharClassValues;
 
 /*
@@ -89,6 +116,7 @@ typedef enum
   SF_REPH_AFTER_BELOW     = 0x40000000,
   SF_EYELASH_RA           = 0x20000000,
   SF_MPRE_FIXUP           = 0x10000000,
+  SF_PROCESS_ZWJ          = 0x08000000,
 
   SF_POST_BASE_LIMIT_MASK = 0x0000FFFF,
   SF_NO_POST_BASE_LIMIT   = 0x00007FFF
@@ -114,7 +142,8 @@ enum indic_glyph_feature_
   blwm = 0x1000,
   abvm = 0x2000,
   dist = 0x4000,
-  junk = 0x8000
+  junk = 0x8000,
+  init = 0x10000
 };
 
 /*
@@ -130,33 +159,13 @@ enum indic_glyph_feature_
  */
 enum indic_glyph_property_
 {
-  rphf_p = (junk | dist),
-  blwf_p = (junk | dist | rphf),
-  half_p = (junk | dist | rphf | blwf),
-  nukt_p = (junk | dist | rphf | blwf | half)
+  rphf_p = (junk | dist | init),
+  blwf_p = (junk | dist | init | rphf),
+  half_p = (junk | dist | init | rphf | blwf),
+  pstf_p = (junk | dist | init | rphf | blwf | half),
+  nukt_p = (junk | dist | init | rphf | blwf | half | pstf)
 };
 
-/*
- * Per-script character ranges
- */
-#define SCRIPT_RANGE_deva 0x0900, 0x0970
-
-#define SCRIPT_RANGE_beng 0x0980, 0x09FA
-
-#define SCRIPT_RANGE_guru 0x0A00, 0x0A74
-
-#define SCRIPT_RANGE_gujr 0x0A80, 0x0AEF
-
-#define SCRIPT_RANGE_orya 0x0B00, 0x0B70
-
-#define SCRIPT_RANGE_taml 0x0B80, 0x0BF2
-
-#define SCRIPT_RANGE_telu 0x0C00, 0x0C6F
-
-#define SCRIPT_RANGE_knda 0x0C80, 0x0CEF
-
-#define SCRIPT_RANGE_mlym 0x0D00, 0x0D6F
-
 /*
  * Macros to test the charClass flags for various things.
  */
@@ -172,6 +181,8 @@ enum indic_glyph_property_
 
 #define IS_VIRAMA(charClass) ((charClass & CF_CLASS_MASK) == CC_VIRAMA)
 
+#define IS_AL_LAKUNA(charClass) ((charClass & CF_CLASS_MASK) == CC_AL_LAKUNA)
+
 #define IS_VATTU(charClass) ((charClass & CF_VATTU) != 0)
 
 #define IS_MATRA(charClass) ((charClass & CF_CLASS_MASK) == CC_DEPENDENT_VOWEL)
@@ -200,22 +211,23 @@ struct _IndicOTClassTable
   gunichar	      lastChar;
   glong  	      worstCaseExpansion;
   IndicOTScriptFlags  scriptFlags;
-  IndicOTCharClass   *charClasses;
 
+  const IndicOTCharClass  *charClasses;
   const IndicOTSplitMatra *splitMatraTable;
 };
 
 typedef struct _IndicOTClassTable IndicOTClassTable;
 
-extern IndicOTClassTable deva_class_table;
-extern IndicOTClassTable beng_class_table;
-extern IndicOTClassTable guru_class_table;
-extern IndicOTClassTable gujr_class_table;
-extern IndicOTClassTable orya_class_table;
-extern IndicOTClassTable taml_class_table;
-extern IndicOTClassTable telu_class_table;
-extern IndicOTClassTable knda_class_table;
-extern IndicOTClassTable mlym_class_table;
+extern const IndicOTClassTable deva_class_table;
+extern const IndicOTClassTable beng_class_table;
+extern const IndicOTClassTable guru_class_table;
+extern const IndicOTClassTable gujr_class_table;
+extern const IndicOTClassTable orya_class_table;
+extern const IndicOTClassTable taml_class_table;
+extern const IndicOTClassTable telu_class_table;
+extern const IndicOTClassTable knda_class_table;
+extern const IndicOTClassTable mlym_class_table;
+extern const IndicOTClassTable sinh_class_table;
 
 const IndicOTSplitMatra *indic_ot_get_split_matra(const IndicOTClassTable *class_table, IndicOTCharClass char_class);
 
@@ -226,6 +238,7 @@ gboolean indic_ot_is_vm_post(const IndicOTClassTable *class_table, gunichar ch);
 gboolean indic_ot_is_consonant(const IndicOTClassTable *class_table, gunichar ch);
 gboolean indic_ot_is_reph(const IndicOTClassTable *class_table, gunichar ch);
 gboolean indic_ot_is_virama(const IndicOTClassTable *class_table, gunichar ch);
+gboolean indic_ot_is_al_lakuna(const IndicOTClassTable *class_table, gunichar ch);
 gboolean indic_ot_is_nukta(const IndicOTClassTable *class_table, gunichar ch);
 gboolean indic_ot_is_vattu(const IndicOTClassTable *class_table, gunichar ch);
 gboolean indic_ot_is_matra(const IndicOTClassTable *class_table, gunichar ch);
@@ -241,7 +254,7 @@ gboolean indic_ot_has_below_base_form(const IndicOTClassTable *class_table, guni
 
 glong indic_ot_find_syllable(const IndicOTClassTable *class_table, const gunichar *chars, glong prev, glong char_count);
 
-glong indic_ot_reorder(const gunichar *chars, const glong *utf8_offsets, glong char_count, const IndicOTClassTable *class_table, gunichar *out_chars, glong *char_indices, gulong *char_tags);
+glong indic_ot_reorder(const gunichar *chars, const glong *utf8_offsets, glong char_count, const IndicOTClassTable *class_table, gunichar *out_chars, glong *char_indices, gulong *char_tags, MPreFixups **outMPreFixups);
 
 #endif /* PANGO_ENABLE_ENGINE */
 
